/*
 * $Id: rb_window.c 351 2006-02-10 15:25:40Z tilman $
 *
 * Copyright (C) 2004 ruby-ecore team (see AUTHORS)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ruby.h>

#include <Ecore.h>
#include <Ecore_X.h>

#define __RB_WINDOW_C
#include "../ecore/rb_ecore.h"
#include "rb_ecore_x.h"
#include "rb_window.h"
#include "rb_cursor.h"

VALUE cWindow;

static void c_mark (RbWindow *w)
{
	if (!NIL_P (w->parent))
		rb_gc_mark (w->parent);
}

static void c_free (RbWindow *w)
{
	if (w->real)
		ecore_x_window_del (w->real);

	free (w);
}

VALUE TO_ECORE_X_WINDOW (VALUE parent, Ecore_X_Window w)
{
	VALUE self;
	RbWindow *window = NULL;

	self = Data_Make_Struct (cWindow, RbWindow, c_mark, free, window);

	window->real = w;
	window->parent = parent;

	rb_obj_call_init (self, 0, NULL);

	return self;
}

static VALUE c_alloc (VALUE klass)
{
	RbWindow *window = NULL;

	return Data_Make_Struct (cWindow, RbWindow, c_mark, c_free, window);
}

static VALUE c_init (int argc, VALUE *argv, VALUE self)
{
	VALUE parent, geom[4];
	RbWindow *p = NULL;
	int i, igeom[4] = {0, 0, 0, 0};
	Ecore_X_Window pwin = 0;

	GET_OBJ (self, RbWindow, win);

	rb_scan_args (argc, argv, "05",
	              &parent, &geom[0], &geom[1], &geom[2], &geom[3]);

	if (!NIL_P (parent)) {
		CHECK_CLASS (parent, cWindow);
		Data_Get_Struct (parent, RbWindow, p);
		pwin = p->real;
	}

	for (i = 0; i < 4; i++)
		if (!NIL_P (geom[i])) {
			Check_Type (geom[i], T_FIXNUM);
			igeom[i] = FIX2INT (geom[i]);
		}

	win->real = ecore_x_window_new (pwin, igeom[0], igeom[1],
	                                igeom[2], igeom[3]);
	win->parent = parent;

	rb_iv_set (self, "@cursor", Qnil);

	return self;
}

static VALUE c_inspect (VALUE self)
{
	char buf[128];

	GET_OBJ (self, RbWindow, win);

	snprintf (buf, sizeof (buf),
	          "#<%s:%p id=%u>", rb_obj_classname (self),
	          (void *) self, win->real);

	return rb_str_new2 (buf);
}

static VALUE c_equal_value (VALUE self, VALUE other)
{
	GET_OBJ (self, RbWindow, w1);

	CHECK_CLASS (other, cWindow);
	GET_OBJ (other, RbWindow, w2);

	return w1->real == w2->real ? Qtrue : Qfalse;
}

static VALUE c_configure (int argc, VALUE *argv, VALUE self)
{
	VALUE mask, geom[4], border, sibling, stackm;
	RbWindow *sibwin = NULL;
	int i;

	GET_OBJ (self, RbWindow, win);

	rb_scan_args (argc, argv, "17", &mask,
	              &geom[0], &geom[1], &geom[2], &geom[3],
	              &border, &sibling, &stackm);

	Check_Type (mask, T_FIXNUM);

	for (i = 0; i < 4; i++)
		if (!NIL_P (geom[i]))
			Check_Type (geom[i], T_FIXNUM);

	if (!NIL_P (border))
		Check_Type (border, T_FIXNUM);

	if (!NIL_P (sibling)) {
		CHECK_CLASS (sibling, cWindow);
		Data_Get_Struct (sibling, RbWindow, sibwin);
	}

	if (!NIL_P (stackm))
		Check_Type (stackm, T_FIXNUM);

	ecore_x_window_configure (win->real, FIX2INT (mask),
	                          NIL_P (geom[0]) ? 0 : FIX2INT (geom[0]),
	                          NIL_P (geom[1]) ? 0 : FIX2INT (geom[1]),
	                          NIL_P (geom[2]) ? 0 : FIX2INT (geom[2]),
	                          NIL_P (geom[3]) ? 0 : FIX2INT (geom[3]),
	                          NIL_P (border) ? 0 : FIX2INT (border),
	                          sibwin ? sibwin->real : 0,
	                          NIL_P (stackm) ? 0 : FIX2INT (stackm));

	return Qnil;
}

static VALUE c_show (VALUE self)
{
	GET_OBJ (self, RbWindow, win);

	ecore_x_window_show (win->real);

	return Qnil;
}

static VALUE c_hide (VALUE self)
{
	GET_OBJ (self, RbWindow, win);

	ecore_x_window_hide (win->real);

	return Qnil;
}

static VALUE c_visible_get (VALUE self)
{
	GET_OBJ (self, RbWindow, win);

	return ecore_x_window_visible_get (win->real) ? Qtrue : Qfalse;
}

static VALUE c_delete (VALUE self)
{
	GET_OBJ (self, RbWindow, win);

	ecore_x_window_del (win->real);

	return Qnil;
}

static VALUE c_send_delete_request (VALUE self)
{
	GET_OBJ (self, RbWindow, win);

	ecore_x_window_delete_request_send (win->real);

	return Qnil;
}

static VALUE c_raise (VALUE self)
{
	GET_OBJ (self, RbWindow, win);

	ecore_x_window_raise (win->real);

	return Qnil;
}

static VALUE c_lower (VALUE self)
{
	GET_OBJ (self, RbWindow, win);

	ecore_x_window_lower (win->real);

	return Qnil;
}

static VALUE c_reparent (VALUE self, VALUE other, VALUE x, VALUE y)
{
	GET_OBJ (self, RbWindow, win);

	CHECK_CLASS (other, cWindow);
	GET_OBJ (other, RbWindow, o);

	Check_Type (x, T_FIXNUM);
	Check_Type (y, T_FIXNUM);

	ecore_x_window_reparent (win->real, o->real,
	                         FIX2INT (x), FIX2INT (y));

	return Qnil;
}

static VALUE c_focus (VALUE self)
{
	GET_OBJ (self, RbWindow, win);

	ecore_x_window_focus (win->real);

	return Qnil;
}

static VALUE c_move (VALUE self, VALUE x, VALUE y)
{
	GET_OBJ (self, RbWindow, win);

	Check_Type (x, T_FIXNUM);
	Check_Type (y, T_FIXNUM);

	ecore_x_window_move (win->real, FIX2INT (x), FIX2INT (y));

	return Qnil;
}

static VALUE c_resize (VALUE self, VALUE w, VALUE h)
{
	GET_OBJ (self, RbWindow, win);

	Check_Type (w, T_FIXNUM);
	Check_Type (h, T_FIXNUM);

	ecore_x_window_resize (win->real, FIX2INT (w), FIX2INT (h));

	return Qnil;
}

static VALUE c_size_get (VALUE self)
{
	int w = 0, h = 0;

	GET_OBJ (self, RbWindow, win);

	ecore_x_window_size_get (win->real, &w, &h);

	return rb_ary_new3 (2, INT2FIX (w), INT2FIX (h));
}

static VALUE c_geometry_get (VALUE self)
{
	int x = 0, y = 0, w = 0, h = 0;

	GET_OBJ (self, RbWindow, win);

	ecore_x_window_geometry_get (win->real, &x, &y, &w, &h);

	return rb_ary_new3 (4, INT2FIX (x), INT2FIX (y),
	                    INT2FIX (w), INT2FIX (h));
}

static VALUE c_border_width_get (VALUE self)
{
	GET_OBJ (self, RbWindow, win);

	return INT2FIX (ecore_x_window_border_width_get (win->real));
}

static VALUE c_border_width_set (VALUE self, VALUE val)
{
	GET_OBJ (self, RbWindow, win);

	Check_Type (val, T_FIXNUM);

	ecore_x_window_border_width_set (win->real, FIX2INT (val));

	return Qnil;
}

static VALUE c_depth_get (VALUE self)
{
	GET_OBJ (self, RbWindow, win);

	return INT2FIX (ecore_x_window_depth_get (win->real));
}

static VALUE c_parent_get (VALUE self)
{
	GET_OBJ (self, RbWindow, win);

	return TO_ECORE_X_WINDOW (Qnil,
	                          ecore_x_window_parent_get (win->real));
}

#if 0
static VALUE c_title_get (VALUE self)
{
	char *s;

	GET_OBJ (self, RbWindow, win);

	s = ecore_x_window_prop_title_get (win->real);

	return s ? rb_str_new2 (s) : Qnil;
}

static VALUE c_title_set (VALUE self, VALUE val)
{
	GET_OBJ (self, RbWindow, win);

	Check_Type (val, T_STRING);

	ecore_x_window_prop_title_set (win->real, StringValuePtr (val));

	return Qnil;
}
#endif

static VALUE c_set_event_mask (VALUE self, VALUE val)
{
	GET_OBJ (self, RbWindow, win);

	Check_Type (val, T_FIXNUM);

	ecore_x_event_mask_set (win->real, FIX2INT (val));

	return Qnil;
}

static VALUE c_unset_event_mask (VALUE self, VALUE val)
{
	GET_OBJ (self, RbWindow, win);

	Check_Type (val, T_FIXNUM);

	ecore_x_event_mask_unset (win->real, FIX2INT (val));

	return Qnil;
}

#if 0
static VALUE c_set_protocol (VALUE self, VALUE proto, VALUE on)
{
	GET_OBJ (self, RbWindow, win);

	Check_Type (proto, T_FIXNUM);
	CHECK_BOOL (on);

	ecore_x_window_prop_protocol_set (win->real, FIX2INT (proto),
	                                  on == Qtrue);

	return Qnil;
}
#endif

static VALUE c_get_protocol (VALUE self, VALUE proto)
{
	int s;

	GET_OBJ (self, RbWindow, win);

	s = ecore_x_window_prop_protocol_isset (win->real, FIX2INT (proto));

	return s ? Qtrue : Qfalse;
}

#if 0
static VALUE c_sticky_get (VALUE self)
{
	int s;

	GET_OBJ (self, RbWindow, win);

	s = ecore_x_window_prop_state_isset (win->real,
	                                     ECORE_X_WINDOW_STATE_STICKY);

	return s ? Qtrue : Qfalse;
}

static VALUE c_sticky_set (VALUE self, VALUE val)
{
	GET_OBJ (self, RbWindow, win);

	CHECK_BOOL (val);

	ecore_x_window_prop_sticky_set (win->real, val == Qtrue);

	return Qnil;
}

static VALUE c_borderless_get (VALUE self)
{
	int s;

	GET_OBJ (self, RbWindow, win);

	s = ecore_x_window_prop_borderless_get (win->real);

	return s ? Qtrue : Qfalse;
}

static VALUE c_borderless_set (VALUE self, VALUE val)
{
	GET_OBJ (self, RbWindow, win);

	CHECK_BOOL (val);

	ecore_x_window_prop_borderless_set (win->real, val == Qtrue);

	return Qnil;
}
#endif

static VALUE c_cursor_set (VALUE self, VALUE val)
{
	GET_OBJ (self, RbWindow, win);

	CHECK_CLASS (val, cCursor);
	GET_OBJ (val, RbCursor, c);

	ecore_x_window_cursor_set (win->real, c->real);
	rb_iv_set (self, "@cursor", val);

	return Qnil;
}

static VALUE c_manage (VALUE self)
{
	GET_OBJ (self, RbWindow, win);

	ecore_x_window_manage (win->real);

	return Qnil;
}

static VALUE c_manage_container (VALUE self)
{
	GET_OBJ (self, RbWindow, win);

	ecore_x_window_container_manage (win->real);

	return Qnil;
}

static VALUE c_manage_client (VALUE self)
{
	GET_OBJ (self, RbWindow, win);

	ecore_x_window_client_manage (win->real);

	return Qnil;
}

static VALUE c_sniff (VALUE self)
{
	GET_OBJ (self, RbWindow, win);

	ecore_x_window_sniff (win->real);

	return Qnil;
}

static VALUE c_sniff_client (VALUE self)
{
	GET_OBJ (self, RbWindow, win);

	ecore_x_window_client_sniff (win->real);

	return Qnil;
}

void Init_Window (void)
{
	cWindow = rb_define_class_under (mX, "Window", rb_cObject);

	rb_define_alloc_func (cWindow, c_alloc);
	rb_define_method (cWindow, "initialize", c_init, -1);
	rb_define_method (cWindow, "inspect", c_inspect, 0);
	rb_define_method (cWindow, "eql?", c_equal_value, 1);
	rb_define_method (cWindow, "==", c_equal_value, 1);
	rb_define_method (cWindow, "===", c_equal_value, 1);
	rb_define_method (cWindow, "configure", c_configure, -1);
	rb_define_method (cWindow, "show", c_show, 0);
	rb_define_method (cWindow, "hide", c_hide, 0);
	rb_define_method (cWindow, "visible?", c_visible_get, 0);
	rb_define_method (cWindow, "delete", c_delete, 0);
	rb_define_method (cWindow, "send_delete_request",
	                  c_send_delete_request, 0);
	rb_define_method (cWindow, "raise", c_raise, 0);
	rb_define_method (cWindow, "lower", c_lower, 0);
	rb_define_method (cWindow, "reparent", c_reparent, 3);
	rb_define_method (cWindow, "focus", c_focus, 0);
	rb_define_method (cWindow, "move", c_move, 2);
	rb_define_method (cWindow, "resize", c_resize, 2);
	rb_define_method (cWindow, "size", c_size_get, 0);
	rb_define_method (cWindow, "geometry", c_geometry_get, 0);
	rb_define_method (cWindow, "border_width", c_border_width_get, 0);
	rb_define_method (cWindow, "border_width=", c_border_width_set, 1);
	rb_define_method (cWindow, "depth", c_depth_get, 0);
	rb_define_method (cWindow, "parent", c_parent_get, 0);
#if 0
	rb_define_method (cWindow, "title", c_title_get, 0);
	rb_define_method (cWindow, "title=", c_title_set, 1);
#endif
	rb_define_method (cWindow, "set_event_mask", c_set_event_mask, 1);
	rb_define_method (cWindow, "unset_event_mask",
	                  c_unset_event_mask, 1);
#if 0
	rb_define_method (cWindow, "set_protocol", c_set_protocol, 2);
#endif
	rb_define_method (cWindow, "get_protocol", c_get_protocol, 1);
#if 0
	rb_define_method (cWindow, "sticky?", c_sticky_get, 0);
	rb_define_method (cWindow, "sticky=", c_sticky_set, 1);
	rb_define_method (cWindow, "borderless?", c_borderless_get, 0);
	rb_define_method (cWindow, "borderless=", c_borderless_set, 1);
#endif
	rb_define_method (cWindow, "cursor=", c_cursor_set, 1);
	rb_define_method (cWindow, "manage", c_manage, 0);
	rb_define_method (cWindow, "manage_container", c_manage_container, 0);
	rb_define_method (cWindow, "manage_client", c_manage_client, 0);
	rb_define_method (cWindow, "sniff", c_sniff, 0);
	rb_define_method (cWindow, "sniff_client", c_sniff_client, 0);

	rb_define_attr (cWindow, "cursor", 1, 0);
}
